Crate orengine_utils

Crate orengine_utils 

This crate provides some useful utilities.

Modules

backoff
This module provides a Backoff that can be used to busy-wait with preemptive yield when it is necessary.
cache_padded
Provides cache-padded atomic types.
cheap_random
Fast non-cryptographic pseudo-random number generators.
hints
Hints to the compiler that affects how code should be emitted or optimized.
light_arc
This module provides LightArc.
numa
Provides utilities for working with NUMA (Non-Uniform Memory Access) nodes.
rw_serde
treap
A randomized binary search tree with subtree-augmented filtering.
varint
Variable-length integer (varint) encoding and decoding.

Macros

config_target_pointer_width_16
Generates code only compiled when the target pointer width is 16 bits.
config_target_pointer_width_32
Generates code only compiled when the target pointer width is 32 bits.
config_target_pointer_width_64
Generates code only compiled when the target pointer width is 64 bits.

Structs

ArrayBuffer
ArrayBuffer is a fixed-sized array-based buffer.
ArrayQueue
ArrayQueue is a queue, but it uses an array on a stack and can’t be resized.
OrengineInstant
A monotone clock. It can be converted to/from std::time::Instant.
SmallString
A UTF-8 string with configurable inline storage.
VecQueue
A queue that uses a vector to store the elements.

Functions

clear_with
This function is like drain for the whole Vec, but faster.