DataPerNUMANodeManager

Struct DataPerNUMANodeManager 

pub struct DataPerNUMANodeManager<T>(/* private fields */);
Manages data per NUMA node. It allows storing data for each NUMA node and accessing it by the NUMA node ID.

Example

use orengine_utils::numa::{DataPerNUMANodeManager, get_current_thread_numa_node};

// Create a manager with data for each NUMA node
static MANAGER: DataPerNUMANodeManager<usize> = DataPerNUMANodeManager::from_arr([0; 64]);

let numa_node = get_current_thread_numa_node();
println!("Memory by {numa_node} NUMA node contains {}", MANAGER.get_ref_by_node(numa_node));

Implementations

Source

impl<T> DataPerNUMANodeManager<T>

pub const fn from_arr(inner: [T; 64]) -> Self

Creates a new manager from an array of data for each NUMA node.

pub fn get_ref_by_node(&self, numa_node: usize) -> &T

Gets a reference to the data for the specified NUMA node.

Panics

Panics if the NUMA node ID is out of bounds.

pub fn get_mut_by_node(&mut self, numa_node: usize) -> &mut T

Gets a mutable reference to the data for the specified NUMA node.

Panics

Panics if the NUMA node ID is out of bounds.

pub fn iter(&self) -> impl Iterator<Item = &T>

Returns an iterator over references to the data for all NUMA nodes.

pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut T>

Returns an iterator over mutable references to the data for all NUMA nodes.

pub fn as_ptr(&self) -> *const [T; 64]

Returns a pointer to the inner array.