BaseTreapEntry

Struct BaseTreapEntry 

pub struct BaseTreapEntry<SortingKey: Ord, FilteringKey: Ord + Clone, V> {
    pub sorting_key: SortingKey,
    pub filtering_key: FilteringKey,
    pub value: V,
}
A ready-to-use TreapEntry implementation wrapping a sorting key, filtering key, and a value.

Use this when you don’t need a custom entry type.

Example

use orengine_utils::treap::{BaseTreapEntry, Treap};

let mut treap = Treap::<BaseTreapEntry<u32, u32, &str>>::new();

treap.set(BaseTreapEntry::new(1, 10, "hello"));

assert_eq!(treap.find(&1).unwrap().value, "hello");

Fields

sorting_key: SortingKeyfiltering_key: FilteringKeyvalue: V

Implementations

Source

impl<SortingKey: Ord, FilteringKey: Ord + Clone, V> BaseTreapEntry<SortingKey, FilteringKey, V>

pub fn new( sorting_key: SortingKey, filtering_key: FilteringKey, value: V, ) -> Self

Creates a new BaseTreapEntry with the given sorting key, filtering key, and value.

Example
use orengine_utils::treap::BaseTreapEntry;

let entry = BaseTreapEntry::new(42u32, 7u32, "payload");

assert_eq!(entry.sorting_key, 42);
assert_eq!(entry.filtering_key, 7);
assert_eq!(entry.value, "payload");